//
//  Message.m
//  MailDelivery
//
//  Created by Dante Palacios on 30/08/09.
//  Copyright 2009 Dante Palacios. All rights reserved.
//


#import <Security/Security.h>

#import "Message.h"
#import "SMTPMailDelivery.h"

#import "NSDictionaryAdditions.h"
#import "NSStringAdditions.h"
#import "NSAttributedStringAdditions.h"
#import "NSDataAdditions.h"

#import "Errors.h"
#import "Constants.h"
#import "Funtions.h"
#import "Macros.h"



@interface Message ()

@property (readwrite, copy) NSString *identifier;
@property (readwrite, retain) NSDate *creationDate;
@property (readonly) NSString *header;
@property (readonly) NSString *body;

@end

@implementation Message


@synthesize representedObject = _representedObject;
@synthesize sender = _sender;
@synthesize toRecipients = _toRecipients;
@synthesize ccRecipients = _ccRecipients;
@synthesize bccRecipients = _bccRecipients;
@synthesize replyTo = _replyTo;
@synthesize subject = _subject;
@synthesize creationDate = _creationDate;
@synthesize dateSent = _dateSent;
@synthesize parts = _parts;
@synthesize source = _source;
@synthesize attachments = _attachments;
@synthesize identifier = _identifier;
@synthesize recipients = _recipients;
@synthesize encoding = _encoding;
@synthesize priority = _priority;


#pragma mark Life Cycle

- (id)init 
{
	self = [super init];
	if (self != nil) 
	{
		self.format = kMultipartAlternativeMessageFormat;
		self.creationDate = [NSDate date];
		self.encoding = NSASCIIStringEncoding;
		self.priority = MessagePriorityNormal;
	}
	return self;
}

- (id)initWithHeaders:(NSDictionary *)messageHeaders content:(NSAttributedString *)messageBody format:(NSString *)messageFormat 
{
	Message *message = [self init];
	if (message) 
	{
		message.headers = messageHeaders;
		message.content = messageBody;
		message.format = messageFormat;
	}
	return message;
}

- (id)initWithHeaders:(NSDictionary *)messageHeaders content:(NSAttributedString *)messageBody 
{
	return [self initWithHeaders:messageHeaders content:messageBody format:kMultipartAlternativeMessageFormat];
}

- (id)copyWithZone:(NSZone *)zone {
	Message *message = [super copyWithZone:zone];
	message.representedObject = self.representedObject;
	message.creationDate = [NSDate date];
	message.encoding = self.encoding;
	message.priority = self.priority;
	
	// no, no, no
	//message.identifier = self.identifier;
	//message.parts = self.parts;
	//message.source = self.source;
	return message;
}

- (void)encodeWithCoder:(NSCoder *)aCoder 
{
	[super encodeWithCoder:aCoder];
	
	if ([self.representedObject conformsToProtocol:@protocol(NSCoding)]) 
	{
		[aCoder encodeObject:self.representedObject forKey:@"representedObject"];
	}
	
	[aCoder encodeObject:self.identifier forKey:@"identifier"];
	[aCoder encodeObject:self.parts forKey:@"parts"];
	[aCoder encodeObject:self.source forKey:@"source"];
	[aCoder encodeInteger:self.encoding forKey:@"encoding"];
}

- (id)initWithCoder:(NSCoder *)aDecoder 
{
	self = [super initWithCoder:aDecoder];
	if (self != nil) 
	{
		self.representedObject = [aDecoder decodeObjectForKey:@"representedObject"];
		self.identifier = [aDecoder decodeObjectForKey:@"identifier"];
		self.parts = [aDecoder decodeObjectForKey:@"parts"];
		self.source = [aDecoder decodeObjectForKey:@"source"];
		self.encoding = [aDecoder decodeIntegerForKey:@"encoding"];
	}
	return self;
}

- (void)dealloc 
{
	[_representedObject release]; 
	_representedObject = nil;
	[_identifier release];
	_identifier = nil;
	[_creationDate release];
	_creationDate = nil;
	[_dateSent release]; 
	_dateSent = nil;
	[_parts release];
	_parts = nil;
	[_source release];
	_source = nil;
	[_attachments release];
	_attachments = nil;
	[_sender release];
	_sender = nil;
	[_subject release];
	_subject = nil;
	[_replyTo release];
	_replyTo = nil;
	[_recipients release];
	_recipients = nil;
	[_toRecipients release];
	_toRecipients = nil;
	[_ccRecipients release];
	_ccRecipients = nil;
	[_bccRecipients release];
	_bccRecipients = nil;
	
	[super dealloc];
}

#pragma mark getters & setters

- (NSDate *)creationDate 
{
    return AtomicAutoreleasedGet(_creationDate);
}

- (void)setCreationDate:(NSDate *)value 
{
    AtomicRetainedSet(_creationDate, value);
}

- (NSDate *)dateSent
{
	return AtomicAutoreleasedGet(_dateSent);
}

- (void)setDateSent:(NSDate *)value
{
	AtomicRetainedSet(_dateSent, value);
}

- (NSArray *)attachments
{
	NSArray *value = nil;
	@synchronized(self)
	{
		if (!_attachments)
		{
			_attachments = [[(NSAttributedString *) self.content attachments] retain];
		}
		value = [[_attachments retain] autorelease];
	}
	return value;
}

- (void)setAttachments:(NSArray *)value
{
     AtomicRetainedSet(_attachments, value);
}

- (NSArray *)parts
{
	NSArray *value = nil;
	
	@synchronized(self)
	{
		if (!_parts)
		{
			
			//at this point the parts still do not have a boundary, moreover, the properties -charset and -contentTransferEncoding must be changed to match the message.
			NSStringEncoding encoding = self.encoding;
			self.charset = [NSString mimeNameOfStringEncoding:encoding];
			self.contentTransferEncoding = (encoding != NSASCIIStringEncoding) ? @"8bit" : @"7bit";
			
			NSString *boundary = self.boundary;
			NSString *relatedPartBoundary = self.relatedBoundary;
			NSArray *availableParts = [(NSAttributedString *)self.content parts];
			NSMutableArray *parts = [NSMutableArray arrayWithCapacity:availableParts.count];
			BOOL hasAttachments = (availableParts.count > 2);
			for (Part *part in availableParts) 
			{
				id content = part.content;
				if ([content isKindOfClass:[NSString class]]) 
					part.contentTransferEncoding = self.contentTransferEncoding;
				
				if ([part.content isKindOfClass:[NSData class]] || ((part.contentType == kHTMLMessageFormat) && hasAttachments))
					boundary = relatedPartBoundary;
				
				part.charset = self.charset;
				part.boundary = boundary;
				
				[parts addObject:part];
			}
			
			_parts = [parts copy];
		}
		
		value = [[_parts copy] autorelease];
	}	
	
	return value;
}

- (void)setParts:(NSArray *)value
{
    AtomicCopiedSet(_parts, value);
}

- (NSString *)header
{
	NSStringEncoding encoding = self.encoding;
	self.charset = [NSString mimeNameOfStringEncoding:encoding];
	self.contentTransferEncoding = (encoding != NSASCIIStringEncoding) ? @"8bit" : @"7bit";
	self.contentType = self.format;
	
	NSMutableString *header = [NSMutableString string];
	[header appendFormat:@"Message-id: %@\r\n", self.identifier];
	
	if (self.format != kMultipartAlternativeMessageFormat)
		[header appendFormat:@"Content-Type: %@; charset=%@; format=flowed\r\n", self.contentType, self.charset];
	else
		[header appendFormat:@"Content-Type: %@; boundary=%@\r\n", self.contentType, self.boundary];
	
	NSString *date = [[NSCalendarDate calendarDate] descriptionWithCalendarFormat:@"%a, %e %b %Y %H:%M:%S %z" locale:[[NSUserDefaults standardUserDefaults] dictionaryRepresentation]];
	
	NSBundle *framework = [NSBundle bundleForClass:[Message class]];
	[header appendFormat:@"Date: %@\r\n", date];
	[header appendFormat:@"Mime-Version: 1.0 (%@ framework v%@)\r\n", [framework objectForInfoDictionaryKey:(NSString *) kCFBundleNameKey], [framework objectForInfoDictionaryKey:(NSString *) kCFBundleVersionKey]];
	
	if ((self.priority > 0) && (self.priority <= MessagePriorityLow) && (self.priority != MessagePriorityNormal)) 
	{
		[header appendFormat:@"X-Priority: %d\r\n", self.priority];
	}
	
	NSDictionary *headers = self.headers;
	NSArray *keys = [[headers allKeys] sortedArrayUsingSelector:@selector(compare:)];
	for (NSString *key in keys) 
	{
		if (!key.length) continue;
		
		NSString *headerName = [key capitalizedString];
		//the value of a header must be a NSString
		NSString *headerValue = nil;
		
		id object = [headers objectForKey:key];
		if ([object isKindOfClass:[NSArray class]]) 
			// this works with objects conforming MessageAddressee protocol
			headerValue = [(NSArray *)object componentsJoinedByString:@", "];
		else if ([object isKindOfClass:[NSString class]]) 
			headerValue = (NSString *)object;
		else if ([object isKindOfClass:[MessageAddressee class]] || [object conformsToProtocol:@protocol(MessageAddressee)]) 
			headerValue = [object fullEmailAddress];//[object description];
		else if ([object isKindOfClass:[NSNumber class]]) 
			headerValue = [object stringValue];
		
		if (![[headerValue stringByTrimmingWhiteSpaces] length]) continue;
		
		//TODO: Check message header line length limits...
		[header appendFormat:@"%@: %@\r\n", headerName, headerValue];
	}
	
	[header appendString:@"\r\n"];
	
	return (NSString *)header;
}

- (NSString *)body
{
	if (self.format != kMultipartAlternativeMessageFormat)
	{
		return [[(NSAttributedString *) self.content plainText] stringWithCRLFLineBreaks];
	}
	
	NSArray *parts = self.parts;
	NSString *boundary = self.boundary;
	NSMutableString *body = [NSMutableString string];
	BOOL hasAttachments = (parts.count > 2);
	for (Part *part in parts) {
		if ((part.contentType == kHTMLMessageFormat) && hasAttachments) 
		{
			[body appendFormat:@"\r\n--%@\r\n", boundary];
			[body appendFormat:@"Content-Type: multipart/related;\r\n\tboundary=%@;\r\n\ttype=\"%@\"\r\n", part.boundary, kHTMLMessageFormat];
		}
		[body appendString:[part description]];
	}
	
	if (hasAttachments)
	{
		[body appendFormat:@"\r\n--%@--\r\n", self.relatedBoundary];
	}
	
	[body appendFormat:@"\r\n--%@--\r\n", boundary];
	
	return (NSString *)body;
}

- (NSString *)source
{
	NSString *value = nil;
	
	@synchronized(self)
	{
		if (!_source)
		{
			NSMutableString *source = [NSMutableString string];
			[source appendString:self.header];
			[source appendString:self.body];
			[source appendString:@"\r\n.\r\n"];
			
			_source = [source copy];
		}
		
		value = [[_source retain] autorelease];
	}
	
	return value;
}

- (void)setSource:(NSString *)value 
{
    AtomicCopiedSet(_source, value);
}

- (NSString *)identifier 
{
	NSString *value = nil;
	@synchronized(self) 
	{
		if (!_identifier) 
		{
			_identifier = [[NSString stringWithUUID] copy];
		}
		value = [[_identifier retain] autorelease];
	}
	
	return value;
}

- (void)setIdentifier:(NSString *)value 
{
    AtomicCopiedSet(_identifier, value);
}

- (void)setHeaders:(NSDictionary *)value
{
	[super setHeaders:value];
	
	@synchronized(self)
	{
		// these values are taken from headers
		[_sender release];
		_sender = nil;
		[_subject release];
		_subject = nil;
		[_replyTo release];
		_replyTo = nil;
		[_recipients release];
		_recipients = nil;
		[_toRecipients release];
		_toRecipients = nil;
		[_ccRecipients release];
		_ccRecipients = nil;
		[_bccRecipients release];
		_bccRecipients = nil;
		
		// reset the source
		[_source release];
		_source = nil;
	}
}

- (id)representedObject
{
    return AtomicAutoreleasedGet(_representedObject);
}

- (void)setRepresentedObject:(id)value
{
    AtomicRetainedSet(_representedObject, value);
}

- (id /*<MessageAddressee>*/)sender
{
	id value = nil;
	@synchronized(self)
	{
		if (!_sender)
		{
			_sender = [[self.headers objectForCaseInsensitiveKey:@"From"] copy];
		}
		
		value = [[_sender copy] autorelease];
	}
	
	return value;
}

- (NSString *)subject
{
	NSString *value = nil;
	@synchronized(self)
	{
		if (!_subject)
		{
			_subject = [[self.headers objectForCaseInsensitiveKey:@"Subject"] retain];
		}
		
		value = [[_subject retain] autorelease];
	}
	
	return value;
}

- (NSArray *)replyTo 
{
    NSArray *value = nil;
	@synchronized(self)
	{
		if (!_replyTo)
		{
			_replyTo = [[self.headers adresseesForMessageHeader:@"Reply-To"] retain];
		}
		
		value = [[_replyTo retain] autorelease];
	}
	
	return value;
}

- (NSArray *)toRecipients
{
	NSArray *value = nil;
	@synchronized(self)
	{
		if (!_toRecipients)
		{
			_toRecipients = [[self.headers adresseesForMessageHeader:@"To"] retain];
		}
		
		value = [[_toRecipients retain] autorelease];
	}
	
	return value;
}

- (NSArray *)ccRecipients
{
	NSArray *value = nil;
	@synchronized(self)
	{
		if (!_ccRecipients)
		{
			_ccRecipients = [[self.headers adresseesForMessageHeader:@"Cc"] retain];
		}
		
		value = [[_ccRecipients retain] autorelease];
	}
	
	return value;
}

- (NSArray *)bccRecipients
{
	NSArray *value = nil;
	@synchronized(self)
	{
		if (!_bccRecipients)
		{
			_bccRecipients = [[self.headers adresseesForMessageHeader:@"Bcc"] retain];
		}
		
		value = [[_bccRecipients retain] autorelease];
	}
	
	return value;
}

- (NSArray *)recipients
{
	NSArray *value = nil;
	@synchronized(self)
	{
		if (!_recipients)
		{
			NSMutableArray *array = [NSMutableArray array];
			[array addObjectsFromArray:self.ccRecipients];
			[array addObjectsFromArray:self.toRecipients];
			[array addObjectsFromArray:self.bccRecipients];
			
			_recipients = [array retain];
		}
		
		value = [[_recipients retain] autorelease];
	}
	
	return value;
}

- (NSStringEncoding)encoding
{
	if (_encoding == NSASCIIStringEncoding)
	{
		_encoding = [[[(NSAttributedString *) self.content plainText] stringWithCRLFLineBreaks] encoding];
	}
	return _encoding;
}
		
@end